package javax.swing.text.html;

import javax.swing.text.ViewFactory;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Position;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics;
import javax.swing.text.StyleConstants;
import javax.swing.text.Element;
import javax.swing.text.AttributeSet;
import javax.swing.text.View;

class HRuleView extends View
{
    private float topMargin;
    private float bottomMargin;
    private float leftMargin;
    private float rightMargin;
    private int alignment;
    private String noshade;
    private int size;
    private CSS.LengthValue widthValue;
    private static final int SPACE_ABOVE = 3;
    private static final int SPACE_BELOW = 3;
    private AttributeSet attr;
    
    public HRuleView(Element elem) {
        super(elem);
        this.alignment = 1;
        this.noshade = null;
        this.size = 0;
        this.setPropertiesFromAttributes();
    }
    
    protected void setPropertiesFromAttributes() {
        StyleSheet styleSheet = ((HTMLDocument)this.getDocument()).getStyleSheet();
        AttributeSet attributes = this.getElement().getAttributes();
        this.attr = styleSheet.getViewAttributes(this);
        this.alignment = 1;
        this.size = 0;
        this.noshade = null;
        this.widthValue = null;
        if (this.attr != null) {
            if (this.attr.getAttribute(StyleConstants.Alignment) != null) {
                this.alignment = StyleConstants.getAlignment(this.attr);
            }
            this.noshade = (String)attributes.getAttribute(HTML.Attribute.NOSHADE);
            Object attribute = attributes.getAttribute(HTML.Attribute.SIZE);
            if (attribute != null && attribute instanceof String) {
                try {
                    this.size = Integer.parseInt((String)attribute);
                }
                catch (NumberFormatException ex) {
                    this.size = 1;
                }
            }
            Object attribute2 = this.attr.getAttribute(CSS.Attribute.WIDTH);
            if (attribute2 != null && attribute2 instanceof CSS.LengthValue) {
                this.widthValue = (CSS.LengthValue)attribute2;
            }
            this.topMargin = this.getLength(CSS.Attribute.MARGIN_TOP, this.attr);
            this.bottomMargin = this.getLength(CSS.Attribute.MARGIN_BOTTOM, this.attr);
            this.leftMargin = this.getLength(CSS.Attribute.MARGIN_LEFT, this.attr);
            this.rightMargin = this.getLength(CSS.Attribute.MARGIN_RIGHT, this.attr);
        }
        else {
            float n = 0.0f;
            this.rightMargin = n;
            this.leftMargin = n;
            this.bottomMargin = n;
            this.topMargin = n;
        }
        this.size = Math.max(2, this.size);
    }
    
    private float getLength(CSS.Attribute attribute, AttributeSet set) {
        CSS.LengthValue lengthValue = (CSS.LengthValue)set.getAttribute(attribute);
        return (lengthValue != null) ? lengthValue.getValue() : 0.0f;
    }
    
    @Override
    public void paint(Graphics graphics, Shape shape) {
        Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
        int n = rectangle.y + 3 + (int)this.topMargin;
        int n2 = rectangle.width - (int)(this.leftMargin + this.rightMargin);
        if (this.widthValue != null) {
            n2 = (int)this.widthValue.getValue((float)n2);
        }
        int size = rectangle.height - (6 + (int)this.topMargin + (int)this.bottomMargin);
        if (this.size > 0) {
            size = this.size;
        }
        int n3;
        switch (this.alignment) {
            case 1: {
                n3 = rectangle.x + rectangle.width / 2 - n2 / 2;
                break;
            }
            case 2: {
                n3 = rectangle.x + rectangle.width - n2 - (int)this.rightMargin;
                break;
            }
            default: {
                n3 = rectangle.x + (int)this.leftMargin;
                break;
            }
        }
        if (this.noshade != null) {
            graphics.setColor(Color.black);
            graphics.fillRect(n3, n, n2, size);
        }
        else {
            Color background = this.getContainer().getBackground();
            Color color;
            Color color2;
            if (background == null || background.equals(Color.white)) {
                color = Color.darkGray;
                color2 = Color.lightGray;
            }
            else {
                color = Color.darkGray;
                color2 = Color.white;
            }
            graphics.setColor(color2);
            graphics.drawLine(n3 + n2 - 1, n, n3 + n2 - 1, n + size - 1);
            graphics.drawLine(n3, n + size - 1, n3 + n2 - 1, n + size - 1);
            graphics.setColor(color);
            graphics.drawLine(n3, n, n3 + n2 - 1, n);
            graphics.drawLine(n3, n, n3, n + size - 1);
        }
    }
    
    @Override
    public float getPreferredSpan(int i) {
        switch (i) {
            case 0: {
                return 1.0f;
            }
            case 1: {
                if (this.size > 0) {
                    return (float)(this.size + 3 + 3) + this.topMargin + this.bottomMargin;
                }
                if (this.noshade != null) {
                    return 8.0f + this.topMargin + this.bottomMargin;
                }
                return 6.0f + this.topMargin + this.bottomMargin;
            }
            default: {
                throw new IllegalArgumentException("Invalid axis: " + i);
            }
        }
    }
    
    @Override
    public int getResizeWeight(int n) {
        if (n == 0) {
            return 1;
        }
        if (n == 1) {
            return 0;
        }
        return 0;
    }
    
    @Override
    public int getBreakWeight(int n, float n2, float n3) {
        if (n == 0) {
            return 3000;
        }
        return 0;
    }
    
    @Override
    public View breakView(int n, int n2, float n3, float n4) {
        return null;
    }
    
    @Override
    public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        if (n >= startOffset && n <= endOffset) {
            Rectangle bounds = shape.getBounds();
            if (n == endOffset) {
                bounds.x += bounds.width;
            }
            bounds.width = 0;
            return bounds;
        }
        else {
            return null;
        }
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
        Rectangle rectangle = (Rectangle)shape;
        if (n < (float)(rectangle.x + rectangle.width / 2)) {
            array[0] = Position.Bias.Forward;
            return this.getStartOffset();
        }
        array[0] = Position.Bias.Backward;
        return this.getEndOffset();
    }
    
    @Override
    public AttributeSet getAttributes() {
        return this.attr;
    }
    
    @Override
    public void changedUpdate(DocumentEvent e, Shape a, ViewFactory f) {
        super.changedUpdate(e, a, f);
        int offset = e.getOffset();
        if (offset <= this.getStartOffset() && offset + e.getLength() >= this.getEndOffset()) {
            this.setPropertiesFromAttributes();
        }
    }
}
